package meta_programming;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.Arrays;

public class Sample19 {
	public static void main(String[] args) {
		ClassLoader classLoader = Thread.currentThread()
				.getContextClassLoader();
		Class<?>[] interfaces = { Foo.class };
		InvocationHandler h = new LoggingInvocationHandler(
				new FooImpl());
		Object object = Proxy.newProxyInstance(classLoader,
				interfaces, h);

		Foo foo = Foo.class.cast(object);
		int result = foo.sum(1, 2);
		System.out.println(result);
	}

	public static interface Foo {
		int sum(int i, int j);
	}

	public static class FooImpl implements Foo {
		public int sum(int i, int j) {
			return i + j;
		}
	}

	public static class LoggingInvocationHandler implements
			InvocationHandler {
		private Object impl;

		public LoggingInvocationHandler(Object impl) {
			this.impl = impl;
		}

		public Object invoke(Object proxy, Method method,
				Object[] args) throws Throwable {
			System.out.println("BEGIN :"
					+ method.getDeclaringClass().getName() + "#"
					+ method.getName() + "("
					+ Arrays.asList(args) + ")");
			Object ret = null;
			try {
				ret = method.invoke(impl, args);
				return ret;
			} finally {
				System.out.println("END   :"
						+ method.getDeclaringClass().getName()
						+ "#" + method.getName() + " : " + ret);
			}
		}
	}
}
